@@ -1,6 +1,6 @@
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## MCE::Core::Input::Handle - File_path and Scalar_ref input reader.
+## File_path and Scalar_ref input reader.
 ##
 ## This package provides the read handle method used internally by the worker
 ## process. Distribution follows a bank-queuing model.
@@ -14,16 +14,16 @@ package MCE::Core::Input::Handle;
 use strict;
 use warnings;
 
-our $VERSION = '1.608';
+our $VERSION = '1.810';
 
 ## Items below are folded into MCE.
 
-package MCE;
+package # hide from rpm
+   MCE;
 
-no warnings 'threads';
-no warnings 'recursion';
-no warnings 'uninitialized';
+no warnings qw( threads recursion uninitialized );
 
+use Fcntl qw( SEEK_CUR );
 use bytes;
 
 my $_que_read_size = $MCE::_que_read_size;
@@ -35,6 +35,10 @@ my $_que_template  = $MCE::_que_template;
 ##
 ###############################################################################
 
+sub _systell {
+   sysseek($_[0], 0, SEEK_CUR);
+}
+
 sub _worker_read_handle {
 
    my ($self, $_proc_type, $_input_data) = @_;
@@ -69,12 +73,8 @@ sub _worker_read_handle {
 
    $_chunk_id  = $_offset_pos = 0;
 
-   if ($_chunk_size <= MAX_RECS_SIZE || $_proc_type == READ_MEMORY) {
-      open    $_IN_FILE, '<', $_input_data or die "$_input_data: $!\n";
-      binmode $_IN_FILE;
-   } else {
-      sysopen $_IN_FILE, $_input_data, O_RDONLY or die "$_input_data: $!\n";
-   }
+   open    $_IN_FILE, '<', $_input_data or die "$_input_data: $!\n";
+   binmode $_IN_FILE;
 
    ## -------------------------------------------------------------------------
 
@@ -107,7 +107,7 @@ sub _worker_read_handle {
       }
 
       ## Read data.
-      if ($_chunk_size <= MAX_RECS_SIZE) {        ## One or many records.
+      if ($_chunk_size <= MAX_RECS_SIZE) {        # One or many records.
          local $/ = $_RS if ($_RS_FLG);
          seek $_IN_FILE, $_offset_pos, 0;
 
@@ -152,71 +152,46 @@ sub _worker_read_handle {
          syswrite $_QUE_W_SOCK,
             pack($_que_template, $_chunk_id, tell $_IN_FILE);
       }
-      else {                                      ## Large chunk.
+      else {                                      # Large chunk.
          local $/ = $_RS if ($_RS_FLG);
 
-         if ($_proc_type == READ_MEMORY) {
-            if ($_parallel_io && ! $_RS_FLG) {
-               syswrite $_QUE_W_SOCK,
-                  pack($_que_template, $_chunk_id, $_offset_pos + $_chunk_size);
+         if ($_parallel_io && ! $_RS_FLG) {
+            syswrite $_QUE_W_SOCK,
+               pack($_que_template, $_chunk_id, $_offset_pos + $_chunk_size);
 
-               $_tmp_cs = $_chunk_size;
-               seek $_IN_FILE, $_offset_pos, 0;
+            $_tmp_cs = $_chunk_size;
+            seek $_IN_FILE, $_offset_pos, 0;
 
-               if ($_offset_pos) {
-                  $_tmp_cs -= length <$_IN_FILE> || 0;
-               }
+            if ($_offset_pos) {
+               $_tmp_cs -= length <$_IN_FILE> || 0;
+            }
 
-               if (read($_IN_FILE, $_, $_tmp_cs, $_p) == $_tmp_cs) {
-                  $_ .= <$_IN_FILE>;
-               }
+            if ($_proc_type == READ_FILE) {
+               sysseek $_IN_FILE, tell( $_IN_FILE ), 0;
+               sysread $_IN_FILE, $_, $_tmp_cs, $_p;
+                  seek $_IN_FILE, _systell( $_IN_FILE ), 0;
             }
             else {
-               seek $_IN_FILE, $_offset_pos, 0;
-
-               if (read($_IN_FILE, $_, $_chunk_size, $_p) == $_chunk_size) {
-                  $_ .= <$_IN_FILE>;
-               }
-
-               syswrite $_QUE_W_SOCK,
-                  pack($_que_template, $_chunk_id, tell $_IN_FILE);
+               read $_IN_FILE, $_, $_tmp_cs, $_p;
             }
+
+            $_ .= <$_IN_FILE>;
          }
          else {
-            if ($_parallel_io && ! $_RS_FLG) {
-               syswrite $_QUE_W_SOCK,
-                  pack($_que_template, $_chunk_id, $_offset_pos + $_chunk_size);
-
-               $_tmp_cs = $_chunk_size;
-
-               if ($_offset_pos) {
-                  seek $_IN_FILE, $_offset_pos, 0;
-                  $_tmp_cs -= length <$_IN_FILE> || 0;
-                  sysseek $_IN_FILE, tell $_IN_FILE, 0;
-               }
-               else {
-                  sysseek $_IN_FILE, $_offset_pos, 0;
-               }
-
-               if (sysread($_IN_FILE, $_, $_tmp_cs, $_p) == $_tmp_cs) {
-                  seek $_IN_FILE, sysseek($_IN_FILE, 0, 1), 0;
-                  $_ .= <$_IN_FILE>;
-               }
+            if ($_proc_type == READ_FILE) {
+               sysseek $_IN_FILE, $_offset_pos, 0;
+               sysread $_IN_FILE, $_, $_chunk_size, $_p;
+                  seek $_IN_FILE, _systell( $_IN_FILE ), 0;
             }
             else {
-               sysseek $_IN_FILE, $_offset_pos, 0;
-
-               if (sysread($_IN_FILE, $_, $_chunk_size, $_p) == $_chunk_size) {
-                  seek $_IN_FILE, sysseek($_IN_FILE, 0, 1), 0;
-                  $_ .= <$_IN_FILE>;
-               }
-               else {
-                  seek $_IN_FILE, sysseek($_IN_FILE, 0, 1), 0;
-               }
-
-               syswrite $_QUE_W_SOCK,
-                  pack($_que_template, $_chunk_id, tell $_IN_FILE);
+               seek $_IN_FILE, $_offset_pos, 0;
+               read $_IN_FILE, $_, $_chunk_size, $_p;
             }
+
+            $_ .= <$_IN_FILE>;
+
+            syswrite $_QUE_W_SOCK,
+               pack($_que_template, $_chunk_id, tell $_IN_FILE);
          }
       }
 
