@@ -1,6 +1,6 @@
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## MCE::Core::Input::Sequence - Sequence of numbers (for task_id == 0).
+## Sequence of numbers (for task_id == 0).
 ##
 ## This package provides a sequence of numbers used internally by the worker
 ## process. Distribution follows a bank-queuing model.
@@ -14,15 +14,14 @@ package MCE::Core::Input::Sequence;
 use strict;
 use warnings;
 
-our $VERSION = '1.608';
+our $VERSION = '1.810';
 
 ## Items below are folded into MCE.
 
-package MCE;
+package # hide from rpm
+   MCE;
 
-no warnings 'threads';
-no warnings 'recursion';
-no warnings 'uninitialized';
+no warnings qw( threads recursion uninitialized );
 
 my $_que_read_size = $MCE::_que_read_size;
 my $_que_template  = $MCE::_que_template;
@@ -95,6 +94,7 @@ sub _worker_sequence_queue {
       if ($_chunk_size == 1) {
          $_ = $_offset * $_step + $_begin;
          $_ = sprintf("%$_fmt", $_) if (defined $_fmt);
+         $_ = [ $_, $_ ] if ($_bounds_only);
          $_wuf->($self, $_, $_chunk_id);
       }
       else {
@@ -104,19 +104,14 @@ sub _worker_sequence_queue {
          ## -------------------------------------------------------------------
 
          if ($_bounds_only) {
-            my $_tmp_b = $_seq_n; my $_tmp_e;
+            my ($_tmp_b, $_tmp_e) = ($_seq_n);
 
             if ($_begin < $_end) {
-               if ($_step * $_chunk_size + $_n_begin <= $_end) {
+               if ($_step * ($_chunk_size - 1) + $_n_begin <= $_end) {
                   $_tmp_e = $_step * ($_chunk_size - 1) + $_n_begin;
                }
                else {
-                  my $_start = int(
-                     ($_end - $_seq_n) / $_chunk_size / $_step * $_chunk_size
-                  );
-                  $_start = 1 if ($_start < 1);
-
-                  for my $_i ($_start .. $_chunk_size) {
+                  for my $_i (1 .. $_chunk_size) {
                      last if ($_seq_n > $_end);
                      $_tmp_e = $_seq_n;
                      $_seq_n = $_step * $_i + $_n_begin;
@@ -124,16 +119,11 @@ sub _worker_sequence_queue {
                }
             }
             else {
-               if ($_step * $_chunk_size + $_n_begin >= $_end) {
+               if ($_step * ($_chunk_size - 1) + $_n_begin >= $_end) {
                   $_tmp_e = $_step * ($_chunk_size - 1) + $_n_begin;
                }
                else {
-                  my $_start = int(
-                     ($_seq_n - $_end) / $_chunk_size / $_step * $_chunk_size
-                  );
-                  $_start = 1 if ($_start < 1);
-
-                  for my $_i ($_start .. $_chunk_size) {
+                  for my $_i (1 .. $_chunk_size) {
                      last if ($_seq_n < $_end);
                      $_tmp_e = $_seq_n;
                      $_seq_n = $_step * $_i + $_n_begin;
@@ -141,11 +131,9 @@ sub _worker_sequence_queue {
                }
             }
 
-            if (defined $_fmt) {
-               @_n = (sprintf("%$_fmt", $_tmp_b), sprintf("%$_fmt", $_tmp_e));
-            } else {
-               @_n = ($_tmp_b, $_tmp_e);
-            }
+            @_n = (defined $_fmt)
+               ? ( sprintf("%$_fmt",$_tmp_b), sprintf("%$_fmt",$_tmp_e) )
+               : ( $_tmp_b, $_tmp_e );
          }
 
          ## -------------------------------------------------------------------
