@@ -1,6 +1,6 @@
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## MCE::Flow - Parallel flow model for building creative applications.
+## Parallel flow model for building creative applications.
 ##
 ###############################################################################
 
@@ -9,92 +9,83 @@ package MCE::Flow;
 use strict;
 use warnings;
 
+no warnings qw( threads recursion uninitialized );
+
+our $VERSION = '1.810';
+
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 use Scalar::Util qw( looks_like_number );
-
 use MCE;
 
-our $VERSION  = '1.608';
-
 our @CARP_NOT = qw( MCE );
 
+my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
+my $_tid = $_has_threads ? threads->tid() : 0;
+
+sub CLONE {
+   $_tid = threads->tid() if $_has_threads;
+}
+
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Import routine.
 ##
 ###############################################################################
 
-my $MAX_WORKERS = 'auto';
-my $CHUNK_SIZE  = 'auto';
-
-my ($_params, @_prev_c, @_prev_n, @_prev_w, @_user_tasks);
-my ($_MCE, $_loaded); my $_tag = 'MCE::Flow';
+my ($_MCE, $_def, $_params, $_tag) = ({}, {}, {}, 'MCE::Flow');
+my ($_prev_c, $_prev_n, $_prev_t, $_prev_w) = ({}, {}, {}, {});
+my ($_user_tasks) = ({});
 
 sub import {
+   my ($_class, $_pkg) = (shift, caller);
 
-   my $_class = shift; return if ($_loaded++);
-
-   ## Process module arguments.
-   while (my $_argument = shift) {
-      my $_arg = lc $_argument;
-
-      $MAX_WORKERS = shift and next if ( $_arg eq 'max_workers' );
-      $CHUNK_SIZE  = shift and next if ( $_arg eq 'chunk_size' );
-
-      $MCE::FREEZE = $MCE::MCE->{freeze} = shift and next
-         if ( $_arg eq 'freeze' );
-      $MCE::THAW = $MCE::MCE->{thaw} = shift and next
-         if ( $_arg eq 'thaw' );
-
-      if ( $_arg eq 'sereal' ) {
-         if (shift eq '1') {
-            local $@; eval 'use Sereal qw(encode_sereal decode_sereal)';
-            unless ($@) {
-               $MCE::FREEZE = $MCE::MCE->{freeze} = \&encode_sereal;
-               $MCE::THAW = $MCE::MCE->{thaw} = \&decode_sereal;
-            }
-         }
-         next;
-      }
-
-      if ( $_arg eq 'tmp_dir' ) {
-         $MCE::TMP_DIR = $MCE::MCE->{tmp_dir} = shift;
-         my $_e1 = 'is not a directory or does not exist';
-         my $_e2 = 'is not writeable';
-         _croak($_tag."::import: ($MCE::TMP_DIR) $_e1") unless -d $MCE::TMP_DIR;
-         _croak($_tag."::import: ($MCE::TMP_DIR) $_e2") unless -w $MCE::TMP_DIR;
-         next;
-      }
-
-      _croak($_tag."::import: ($_argument) is not a valid module argument");
-   }
-
-   $MAX_WORKERS = MCE::Util::_parse_max_workers($MAX_WORKERS);
-   _validate_number($MAX_WORKERS, 'MAX_WORKERS');
-
-   _validate_number($CHUNK_SIZE, 'CHUNK_SIZE')
-      unless ($CHUNK_SIZE eq 'auto');
+   my $_p = $_def->{$_pkg} = {
+      MAX_WORKERS => 'auto',
+      CHUNK_SIZE  => 'auto',
+   };
 
    ## Import functions.
    no strict 'refs'; no warnings 'redefine';
-   my $_pkg = caller;
 
    *{ $_pkg.'::mce_flow_f' } = \&run_file;
    *{ $_pkg.'::mce_flow_s' } = \&run_seq;
    *{ $_pkg.'::mce_flow'   } = \&run;
 
+   ## Process module arguments.
+   while ( my $_argument = shift ) {
+      my $_arg = lc $_argument;
+
+      $_p->{MAX_WORKERS} = shift, next if ( $_arg eq 'max_workers' );
+      $_p->{CHUNK_SIZE}  = shift, next if ( $_arg eq 'chunk_size' );
+      $_p->{TMP_DIR}     = shift, next if ( $_arg eq 'tmp_dir' );
+      $_p->{FREEZE}      = shift, next if ( $_arg eq 'freeze' );
+      $_p->{THAW}        = shift, next if ( $_arg eq 'thaw' );
+
+      ## Sereal 3.008+, if available, is used automatically by MCE 1.800.
+      if ( $_arg eq 'sereal' ) {
+         if ( shift eq '0' ) {
+            require Storable;
+            $_p->{FREEZE} = \&Storable::freeze;
+            $_p->{THAW}   = \&Storable::thaw;
+         }
+         next;
+      }
+
+      _croak("Error: ($_argument) invalid module option");
+   }
+
+   $_p->{MAX_WORKERS} = MCE::Util::_parse_max_workers($_p->{MAX_WORKERS});
+
+   _validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS');
+   _validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE')
+      unless ($_p->{CHUNK_SIZE} eq 'auto');
+
    return;
 }
 
-END {
-   return if (defined $_MCE && $_MCE->wid);
-
-   finish();
-}
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Init and finish routines.
@@ -104,27 +95,35 @@ END {
 sub init (@) {
 
    shift if (defined $_[0] && $_[0] eq 'MCE::Flow');
+   my $_pkg = "$$.$_tid.".caller();
 
-   if (MCE->wid) {
-      @_ = (); _croak(
-         "$_tag: function cannot be called by the worker process"
-      );
-   }
-
-   finish(); $_params = (ref $_[0] eq 'HASH') ? shift : { @_ };
+   $_params->{$_pkg} = (ref $_[0] eq 'HASH') ? shift : { @_ };
 
    @_ = ();
 
    return;
 }
 
-sub finish () {
+sub finish (@) {
 
-   if (defined $_MCE && $_MCE->{_spawned}) {
-      MCE::_save_state; $_MCE->shutdown(); MCE::_restore_state;
+   shift if (defined $_[0] && $_[0] eq 'MCE::Flow');
+   my $_pkg = (defined $_[0]) ? shift : "$$.$_tid.".caller();
+
+   if ( $_pkg eq 'MCE' ) {
+      for my $_k ( keys %{ $_MCE } ) { MCE::Flow->finish($_k, 1); }
+   }
+   elsif ( $_MCE->{$_pkg} && $_MCE->{$_pkg}{_init_pid} eq "$$.$_tid" ) {
+      $_MCE->{$_pkg}->shutdown(@_) if $_MCE->{$_pkg}{_spawned};
+
+      delete $_user_tasks->{$_pkg};
+      delete $_prev_c->{$_pkg};
+      delete $_prev_n->{$_pkg};
+      delete $_prev_t->{$_pkg};
+      delete $_prev_w->{$_pkg};
+      delete $_MCE->{$_pkg};
    }
 
-   @_user_tasks = (); @_prev_w = (); @_prev_n = (); @_prev_c = ();
+   @_ = ();
 
    return;
 }
@@ -140,31 +139,32 @@ sub run_file (@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Flow');
 
    my ($_file, $_pos); my $_start_pos = (ref $_[0] eq 'HASH') ? 2 : 1;
+   my $_pid = "$$.$_tid.".caller();
 
-   if (defined $_params) {
-      delete $_params->{input_data} if (exists $_params->{input_data});
-      delete $_params->{sequence}   if (exists $_params->{sequence});
+   if (defined (my $_p = $_params->{$_pid})) {
+      delete $_p->{input_data} if (exists $_p->{input_data});
+      delete $_p->{sequence}   if (exists $_p->{sequence});
    }
    else {
-      $_params = {};
+      $_params->{$_pid} = {};
    }
 
    for my $_i ($_start_pos .. @_ - 1) {
       my $_r = ref $_[$_i];
-      if ($_r eq '' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
+      if ($_r eq '' || $_r eq 'SCALAR' || $_r =~ /^(?:GLOB|FileHandle|IO::)/) {
          $_file = $_[$_i]; $_pos = $_i;
          last;
       }
    }
 
    if (defined $_file && ref $_file eq '' && $_file ne '') {
-      _croak("$_tag: ($_file) does not exist") unless (-e $_file);
-      _croak("$_tag: ($_file) is not readable") unless (-r $_file);
+      _croak("$_tag: ($_file) does not exist")      unless (-e $_file);
+      _croak("$_tag: ($_file) is not readable")     unless (-r $_file);
       _croak("$_tag: ($_file) is not a plain file") unless (-f $_file);
-      $_params->{_file} = $_file;
+      $_params->{$_pid}{_file} = $_file;
    }
-   elsif (ref $_file eq 'GLOB' || ref $_file eq 'SCALAR' || ref($_file) =~ /^IO::/) {
-      $_params->{_file} = $_file;
+   elsif (ref $_file eq 'SCALAR' || ref($_file) =~ /^(?:GLOB|FileHandle|IO::)/) {
+      $_params->{$_pid}{_file} = $_file;
    }
    else {
       _croak("$_tag: (file) is not specified or valid");
@@ -188,14 +188,15 @@ sub run_seq (@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Flow');
 
    my ($_begin, $_end, $_pos); my $_start_pos = (ref $_[0] eq 'HASH') ? 2 : 1;
+   my $_pid = "$$.$_tid.".caller();
 
-   if (defined $_params) {
-      delete $_params->{sequence}   if (exists $_params->{sequence});
-      delete $_params->{input_data} if (exists $_params->{input_data});
-      delete $_params->{_file}      if (exists $_params->{_file});
+   if (defined (my $_p = $_params->{$_pid})) {
+      delete $_p->{sequence}   if (exists $_p->{sequence});
+      delete $_p->{input_data} if (exists $_p->{input_data});
+      delete $_p->{_file}      if (exists $_p->{_file});
    }
    else {
-      $_params = {};
+      $_params->{$_pid} = {};
    }
 
    for my $_i ($_start_pos .. @_ - 1) {
@@ -206,17 +207,17 @@ sub run_seq (@) {
 
          if ($_ref eq '') {
             $_begin = $_[$_pos]; $_end = $_[$_pos + 1];
-            $_params->{sequence} = [
+            $_params->{$_pid}{sequence} = [
                $_[$_pos], $_[$_pos + 1], $_[$_pos + 2], $_[$_pos + 3]
             ];
          }
          elsif ($_ref eq 'HASH') {
             $_begin = $_[$_pos]->{begin}; $_end = $_[$_pos]->{end};
-            $_params->{sequence} = $_[$_pos];
+            $_params->{$_pid}{sequence} = $_[$_pos];
          }
          elsif ($_ref eq 'ARRAY') {
             $_begin = $_[$_pos]->[0]; $_end = $_[$_pos]->[1];
-            $_params->{sequence} = $_[$_pos];
+            $_params->{$_pid}{sequence} = $_[$_pos];
          }
 
          last;
@@ -224,15 +225,13 @@ sub run_seq (@) {
    }
 
    _croak("$_tag: (sequence) is not specified or valid")
-      unless (exists $_params->{sequence});
-
+      unless (exists $_params->{$_pid}{sequence});
    _croak("$_tag: (begin) is not specified for sequence")
       unless (defined $_begin);
-
    _croak("$_tag: (end) is not specified for sequence")
       unless (defined $_end);
 
-   $_params->{sequence_run} = 1;
+   $_params->{$_pid}{sequence_run} = 1;
 
    if (defined $_pos) {
       pop @_ for ($_pos .. @_ - 1);
@@ -251,16 +250,13 @@ sub run (@) {
 
    shift if (defined $_[0] && $_[0] eq 'MCE::Flow');
 
-   if (MCE->wid) {
-      @_ = (); _croak(
-         "$_tag: function cannot be called by the worker process"
-      );
-   }
+   my $_pkg = caller() eq 'MCE::Flow' ? caller(1) : caller();
+   my $_pid = "$$.$_tid.$_pkg";
 
    if (ref $_[0] eq 'HASH') {
-      $_params = {} unless defined $_params;
+      $_params->{$_pid} = {} unless defined $_params->{$_pid};
       for my $_p (keys %{ $_[0] }) {
-         $_params->{$_p} = $_[0]->{$_p};
+         $_params->{$_pid}{$_p} = $_[0]->{$_p};
       }
 
       shift;
@@ -268,36 +264,42 @@ sub run (@) {
 
    ## -------------------------------------------------------------------------
 
-   my (@_code, @_name, @_wrks); my $_init_mce = 0; my $_pos = 0;
+   my (@_code, @_name, @_thrs, @_wrks); my $_init_mce = 0; my $_pos = 0;
 
    while (ref $_[0] eq 'CODE') {
       push @_code, $_[0];
 
-      push @_name, (defined $_params && ref $_params->{task_name} eq 'ARRAY')
-         ? $_params->{task_name}->[$_pos] : undef;
-      push @_wrks, (defined $_params && ref $_params->{max_workers} eq 'ARRAY')
-         ? $_params->{max_workers}->[$_pos] : undef;
-
-      $_init_mce = 1
-         if (!defined $_prev_c[$_pos] || $_prev_c[$_pos] != $_code[$_pos]);
-
-      {
-         no warnings;
-         $_init_mce = 1 if ($_prev_n[$_pos] ne $_name[$_pos]);
-         $_init_mce = 1 if ($_prev_w[$_pos] ne $_wrks[$_pos]);
+      if (defined (my $_p = $_params->{$_pid})) {
+         push @_name, (ref $_p->{task_name} eq 'ARRAY')
+            ? $_p->{task_name}->[$_pos] : undef;
+         push @_thrs, (ref $_p->{use_threads} eq 'ARRAY')
+            ? $_p->{use_threads}->[$_pos] : undef;
+         push @_wrks, (ref $_p->{max_workers} eq 'ARRAY')
+            ? $_p->{max_workers}->[$_pos] : undef;
       }
 
-      $_prev_c[$_pos] = $_code[$_pos];
-      $_prev_n[$_pos] = $_name[$_pos];
-      $_prev_w[$_pos] = $_wrks[$_pos];
+      $_init_mce = 1 if (
+         !defined $_prev_c->{$_pid}[$_pos] ||
+         $_prev_c->{$_pid}[$_pos] != $_code[$_pos]
+      );
+
+      $_init_mce = 1 if ($_prev_n->{$_pid}[$_pos] ne $_name[$_pos]);
+      $_init_mce = 1 if ($_prev_t->{$_pid}[$_pos] ne $_thrs[$_pos]);
+      $_init_mce = 1 if ($_prev_w->{$_pid}[$_pos] ne $_wrks[$_pos]);
+
+      $_prev_c->{$_pid}[$_pos] = $_code[$_pos];
+      $_prev_n->{$_pid}[$_pos] = $_name[$_pos];
+      $_prev_t->{$_pid}[$_pos] = $_thrs[$_pos];
+      $_prev_w->{$_pid}[$_pos] = $_wrks[$_pos];
 
       shift; $_pos++;
    }
 
-   if (defined $_prev_c[$_pos]) {
-      pop @_prev_c for ($_pos .. @_prev_c - 1);
-      pop @_prev_n for ($_pos .. @_prev_n - 1);
-      pop @_prev_w for ($_pos .. @_prev_w - 1);
+   if (defined $_prev_c->{$_pid}[$_pos]) {
+      pop @{ $_prev_c->{$_pid} } for ($_pos .. $#{ $_prev_c->{$_pid } });
+      pop @{ $_prev_n->{$_pid} } for ($_pos .. $#{ $_prev_n->{$_pid } });
+      pop @{ $_prev_t->{$_pid} } for ($_pos .. $#{ $_prev_t->{$_pid } });
+      pop @{ $_prev_w->{$_pid} } for ($_pos .. $#{ $_prev_w->{$_pid } });
 
       $_init_mce = 1;
    }
@@ -306,13 +308,14 @@ sub run (@) {
 
    ## -------------------------------------------------------------------------
 
-   my $_input_data; my $_max_workers = $MAX_WORKERS; my $_r = ref $_[0];
+   my $_input_data; my $_max_workers = $_def->{$_pkg}{MAX_WORKERS};
+   my $_r = ref $_[0];
 
-   if ($_r eq 'ARRAY' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
-      $_input_data = shift;
+   if ($_r eq 'ARRAY' || $_r eq 'SCALAR' || $_r =~ /^(?:GLOB|FileHandle|IO::)/) {
+      $_input_data = shift if (@_ == 1);
    }
 
-   if (defined $_params) { my $_p = $_params;
+   if (defined (my $_p = $_params->{$_pid})) {
       $_max_workers = MCE::Util::_parse_max_workers($_p->{max_workers})
          if (exists $_p->{max_workers} && ref $_p->{max_workers} ne 'ARRAY');
 
@@ -326,102 +329,108 @@ sub run (@) {
    }
 
    my $_chunk_size = MCE::Util::_parse_chunk_size(
-      $CHUNK_SIZE, $_max_workers, $_params, $_input_data, scalar @_
+      $_def->{$_pkg}{CHUNK_SIZE}, $_max_workers, $_params->{$_pid},
+      $_input_data, scalar @_
    );
 
-   if (defined $_params) {
-      if (exists $_params->{_file}) {
-         $_input_data = delete $_params->{_file};
-      }
-      else {
-         $_input_data = $_params->{input_data} if exists $_params->{input_data};
+   if (defined (my $_p = $_params->{$_pid})) {
+      if (exists $_p->{_file}) {
+         $_input_data = delete $_p->{_file};
+      } else {
+         $_input_data = $_p->{input_data} if exists $_p->{input_data};
       }
    }
 
-   MCE::_save_state;
-
    ## -------------------------------------------------------------------------
 
-   if ($_init_mce) {
-      $_MCE->shutdown() if (defined $_MCE);
-      _gen_user_tasks(\@_code, \@_name, \@_wrks);
+   MCE::_save_state();
 
-      my %_options = (
+   if ($_init_mce) {
+      $_MCE->{$_pid}->shutdown() if (defined $_MCE->{$_pid});
+
+      ## must clear arrays for nested session to work with Perl < v5.14
+      _gen_user_tasks($_pid, [@_code], [@_name], [@_thrs], [@_wrks]);
+
+      @_code = @_name = @_thrs = @_wrks = ();
+
+      my %_opts = (
          max_workers => $_max_workers, task_name => $_tag,
-         user_tasks => \@_user_tasks,
+         user_tasks  => $_user_tasks->{$_pid},
       );
 
-      if (defined $_params) {
-         local $_; my $_p = $_params;
+      if (defined (my $_p = $_params->{$_pid})) {
+         local $_;
 
          for (keys %{ $_p }) {
-            next if ($_ eq 'sequence_run');
             next if ($_ eq 'max_workers' && ref $_p->{max_workers} eq 'ARRAY');
-            next if ($_ eq 'task_name' && ref $_p->{task_name} eq 'ARRAY');
-            next if ($_ eq 'input_data');
+            next if ($_ eq 'task_name'   && ref $_p->{task_name}   eq 'ARRAY');
+            next if ($_ eq 'use_threads' && ref $_p->{use_threads} eq 'ARRAY');
+
             next if ($_ eq 'chunk_size');
+            next if ($_ eq 'input_data');
+            next if ($_ eq 'sequence_run');
 
             _croak("MCE::Flow: ($_) is not a valid constructor argument")
                unless (exists $MCE::_valid_fields_new{$_});
 
-            $_options{$_} = $_p->{$_};
+            $_opts{$_} = $_p->{$_};
          }
       }
 
-      $_MCE = MCE->new(%_options);
+      for my $_k (qw/ tmp_dir freeze thaw /) {
+         $_opts{$_k} = $_def->{$_pkg}{uc($_k)}
+            if (exists $_def->{$_pkg}{uc($_k)} && !exists $_opts{$_k});
+      }
+
+      $_MCE->{$_pid} = MCE->new(pkg => $_pkg, %_opts);
    }
    else {
       ## Workers may persist after running. Thus, updating the MCE instance.
       ## These options do not require respawning.
-      if (defined $_params) {
-         for my $_p (qw(
+      if (defined (my $_p = $_params->{$_pid})) {
+         for my $_k (qw(
             RS interval stderr_file stdout_file user_error user_output
             job_delay submit_delay on_post_exit on_post_run user_args
-            flush_file flush_stderr flush_stdout gather
+            flush_file flush_stderr flush_stdout gather max_retries
          )) {
-            $_MCE->{$_p} = $_params->{$_p} if (exists $_params->{$_p});
+            $_MCE->{$_pid}{$_k} = $_p->{$_k} if (exists $_p->{$_k});
          }
       }
    }
 
    ## -------------------------------------------------------------------------
 
-   my @_a; my $_wa = wantarray; $_MCE->{gather} = \@_a if (defined $_wa);
+   my @_a; my $_wa = wantarray; $_MCE->{$_pid}{gather} = \@_a if (defined $_wa);
 
    if (defined $_input_data) {
       @_ = ();
-      $_MCE->process({ chunk_size => $_chunk_size }, $_input_data);
-      delete $_MCE->{input_data};
+      $_MCE->{$_pid}->process({ chunk_size => $_chunk_size }, $_input_data);
+      delete $_MCE->{$_pid}{input_data};
    }
    elsif (scalar @_) {
-      $_MCE->process({ chunk_size => $_chunk_size }, \@_);
-      delete $_MCE->{input_data};
+      $_MCE->{$_pid}->process({ chunk_size => $_chunk_size }, \@_);
+      delete $_MCE->{$_pid}{input_data};
    }
    else {
-      if (defined $_params && exists $_params->{sequence}) {
-         $_MCE->run({
-            chunk_size => $_chunk_size, sequence => $_params->{sequence}
+      if (defined $_params->{$_pid} && exists $_params->{$_pid}{sequence}) {
+         $_MCE->{$_pid}->run({
+             chunk_size => $_chunk_size,
+             sequence   => $_params->{$_pid}{sequence}
          }, 0);
-         if (exists $_params->{sequence_run}) {
-            delete $_params->{sequence_run};
-            delete $_params->{sequence};
+         if (exists $_params->{$_pid}{sequence_run}) {
+             delete $_params->{$_pid}{sequence_run};
+             delete $_params->{$_pid}{sequence};
          }
-         delete $_MCE->{sequence};
+         delete $_MCE->{$_pid}{sequence};
       }
       else {
-         $_MCE->run({ chunk_size => $_chunk_size }, 0);
+         $_MCE->{$_pid}->run({ chunk_size => $_chunk_size }, 0);
       }
    }
 
-   delete $_MCE->{gather} if (defined $_wa);
+   MCE::_restore_state();
 
-   MCE::_restore_state;
-
-   if (exists $_MCE->{_rla_return}) {
-      $MCE::MCE->{_rla_return} = delete $_MCE->{_rla_return};
-   }
-
-   finish() if ($^S);   ## shutdown if in eval state
+   delete $_MCE->{$_pid}{gather} if (defined $_wa);
 
    return ((defined $_wa) ? @_a : ());
 }
@@ -439,13 +448,14 @@ sub _croak {
 
 sub _gen_user_tasks {
 
-   my ($_code_ref, $_name_ref, $_wrks_ref) = @_;
+   my ($_pid, $_code_ref, $_name_ref, $_thrs_ref, $_wrks_ref) = @_;
 
-   @_user_tasks = ();
+   @{ $_user_tasks->{$_pid} } = ();
 
    for (my $_i = 0; $_i < @{ $_code_ref }; $_i++) {
-      push @_user_tasks, {
+      push @{ $_user_tasks->{$_pid} }, {
          task_name   => $_name_ref->[$_i],
+         use_threads => $_thrs_ref->[$_i],
          max_workers => $_wrks_ref->[$_i],
          user_func   => $_code_ref->[$_i]
       }
@@ -485,7 +495,7 @@ MCE::Flow - Parallel flow model for building creative applications
 
 =head1 VERSION
 
-This document describes MCE::Flow version 1.608
+This document describes MCE::Flow version 1.810
 
 =head1 DESCRIPTION
 
@@ -601,8 +611,8 @@ between sub-tasks. Thus, the least overhead.
    }
 
 In summary, MCE::Flow builds out a MCE instance behind the scene and starts
-running. Both task_name and max_workers (not shown) can take an anonymous
-array for specifying the values uniquely for each sub-task.
+running. The task_name (shown), max_workers, and use_threads options can take
+an anonymous array for specifying the values uniquely per each sub-task.
 
    my @a;
 
@@ -616,7 +626,7 @@ array for specifying the values uniquely for each sub-task.
 
 If speed is not a concern and wanting to rid of all the MCE->freeze and
 MCE->thaw statements, simply enqueue and dequeue 2 items at a time.
-Or better yet, see L<MCE::Step|MCE::Step> introduced in MCE 1.506.
+Or better yet, see L<MCE::Step> introduced in MCE 1.506.
 
 First, task_end must be updated. The number of undef(s) must match the number
 of workers times the dequeue count. Otherwise, the script will stall.
@@ -687,9 +697,8 @@ Finally, run as usual.
 
 =head1 SYNOPSIS when CHUNK_SIZE EQUALS 1
 
-Although L<MCE::Loop|MCE::Loop> may be preferred for running using a single
-code block, the text below also applies to this module, particularly for the
-first block.
+Although L<MCE::Loop> may be preferred for running using a single code block,
+the text below also applies to this module, particularly for the first block.
 
 All models in MCE default to 'auto' for chunk_size. The arguments for the block
 are the same as writing a user_func block using the Core API.
@@ -770,32 +779,24 @@ choosing 1 for chunk_size is fine.
 
 =head1 OVERRIDING DEFAULTS
 
-The following list 5 options which may be overridden when loading the module.
+The following list options which may be overridden when loading the module.
 
    use Sereal qw( encode_sereal decode_sereal );
    use CBOR::XS qw( encode_cbor decode_cbor );
    use JSON::XS qw( encode_json decode_json );
 
    use MCE::Flow
-         max_workers => 8,               ## Default 'auto'
-         chunk_size => 500,              ## Default 'auto'
-         tmp_dir => "/path/to/app/tmp",  ## $MCE::Signal::tmp_dir
-         freeze => \&encode_sereal,      ## \&Storable::freeze
-         thaw => \&decode_sereal         ## \&Storable::thaw
+       max_workers => 8,                # Default 'auto'
+       chunk_size => 500,               # Default 'auto'
+       tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
+       freeze => \&encode_sereal,       # \&Storable::freeze
+       thaw => \&decode_sereal          # \&Storable::thaw
    ;
 
-There is a simpler way to enable Sereal with MCE 1.5. The following will
-attempt to use Sereal if available, otherwise defaults to Storable for
-serialization.
+From MCE 1.8 onwards, Sereal 3.008+ is loaded automatically if available.
+Specify C<Sereal => 0> to use Storable instead.
 
-   use MCE::Flow Sereal => 1;
-
-   MCE::Flow::init {
-      chunk_size => 1
-   };
-
-   ## Serialization is by the Sereal module if available.
-   my %answer = mce_flow sub { MCE->gather( $_, sqrt $_ ) }, 1..10000;
+   use MCE::Flow Sereal => 0;
 
 =head1 CUSTOMIZING MCE
 
@@ -850,18 +851,20 @@ both gather and bounds_only options may be specified when calling init
 =back
 
 Like with MCE::Flow::init above, MCE options may be specified using an
-anonymous hash for the first argument. Notice how both max_workers and
-task_name can take an anonymous array for setting values uniquely
-for each code block.
+anonymous hash for the first argument. Notice how task_name, max_workers,
+and use_threads can take an anonymous array for setting uniquely per
+each code block.
 
 Unlike MCE::Stream which processes from right-to-left, MCE::Flow begins
 with the first code block, thus processing from left-to-right.
 
+   use threads;
    use MCE::Flow;
 
    my @a = mce_flow {
       task_name   => [ 'a', 'b', 'c' ],
       max_workers => [  3,   4,   2, ],
+      use_threads => [  1,   0,   0, ],
 
       user_end => sub {
          my ($mce, $task_id, $task_name) = @_;
@@ -908,7 +911,7 @@ An iterator reference can by specified for input_data. The only other way
 is to specify input_data via MCE::Flow::init. This prevents MCE::Flow from
 configuring the iterator reference as another user task which will not work.
 
-Iterators are described under "SYNTAX for INPUT_DATA" at L<MCE::Core|MCE::Core>.
+Iterators are described under "SYNTAX for INPUT_DATA" at L<MCE::Core>.
 
    MCE::Flow::init {
       input_data => iterator
@@ -953,8 +956,6 @@ optional. The format is passed to sprintf (% may be omitted below).
       begin => $beg, end => $end, step => $step, format => $fmt
    };
 
-=back
-
 The sequence engine can compute 'begin' and 'end' items only, for the chunk,
 and not the items in between (hence boundaries only). This option applies
 to sequence only and has no effect when chunk_size equals 1.
@@ -1007,6 +1008,8 @@ Time was measured using 1 worker to emphasize the difference.
    7500001 ..  8750000
    8750001 .. 10000000
 
+=back
+
 =head1 GATHERING DATA
 
 Unlike MCE::Map where gather and output order are done for you automatically,
@@ -1214,7 +1217,7 @@ longer needed.
 
 =head1 INDEX
 
-L<MCE|MCE>
+L<MCE|MCE>, L<MCE::Core>
 
 =head1 AUTHOR
 
