@@ -1,6 +1,6 @@
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## MCE::Grep - Parallel grep model similar to the native grep function.
+## Parallel grep model similar to the native grep function.
 ##
 ###############################################################################
 
@@ -9,91 +9,81 @@ package MCE::Grep;
 use strict;
 use warnings;
 
+no warnings qw( threads recursion uninitialized );
+
+our $VERSION = '1.810';
+
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
-use Scalar::Util qw( looks_like_number );
-
+use Scalar::Util qw( looks_like_number weaken );
 use MCE;
 
-our $VERSION  = '1.608';
-
 our @CARP_NOT = qw( MCE );
 
+my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
+my $_tid = $_has_threads ? threads->tid() : 0;
+
+sub CLONE {
+   $_tid = threads->tid() if $_has_threads;
+}
+
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Import routine.
 ##
 ###############################################################################
 
-my $MAX_WORKERS = 'auto';
-my $CHUNK_SIZE  = 'auto';
-
-my ($_MCE, $_loaded); my ($_params, $_prev_c); my $_tag = 'MCE::Grep';
+my ($_MCE, $_def, $_params, $_prev_c, $_tag) = ({}, {}, {}, {}, 'MCE::Grep');
 
 sub import {
+   my ($_class, $_pkg) = (shift, caller);
 
-   my $_class = shift; return if ($_loaded++);
-
-   ## Process module arguments.
-   while (my $_argument = shift) {
-      my $_arg = lc $_argument;
-
-      $MAX_WORKERS = shift and next if ( $_arg eq 'max_workers' );
-      $CHUNK_SIZE  = shift and next if ( $_arg eq 'chunk_size' );
-
-      $MCE::FREEZE = $MCE::MCE->{freeze} = shift and next
-         if ( $_arg eq 'freeze' );
-      $MCE::THAW = $MCE::MCE->{thaw} = shift and next
-         if ( $_arg eq 'thaw' );
-
-      if ( $_arg eq 'sereal' ) {
-         if (shift eq '1') {
-            local $@; eval 'use Sereal qw(encode_sereal decode_sereal)';
-            unless ($@) {
-               $MCE::FREEZE = $MCE::MCE->{freeze} = \&encode_sereal;
-               $MCE::THAW = $MCE::MCE->{thaw} = \&decode_sereal;
-            }
-         }
-         next;
-      }
-
-      if ( $_arg eq 'tmp_dir' ) {
-         $MCE::TMP_DIR = $MCE::MCE->{tmp_dir} = shift;
-         my $_e1 = 'is not a directory or does not exist';
-         my $_e2 = 'is not writeable';
-         _croak($_tag."::import: ($MCE::TMP_DIR) $_e1") unless -d $MCE::TMP_DIR;
-         _croak($_tag."::import: ($MCE::TMP_DIR) $_e2") unless -w $MCE::TMP_DIR;
-         next;
-      }
-
-      _croak($_tag."::import: ($_argument) is not a valid module argument");
-   }
-
-   $MAX_WORKERS = MCE::Util::_parse_max_workers($MAX_WORKERS);
-   _validate_number($MAX_WORKERS, 'MAX_WORKERS');
-
-   _validate_number($CHUNK_SIZE, 'CHUNK_SIZE')
-      unless ($CHUNK_SIZE eq 'auto');
+   my $_p = $_def->{$_pkg} = {
+      MAX_WORKERS => 'auto',
+      CHUNK_SIZE  => 'auto',
+   };
 
    ## Import functions.
    no strict 'refs'; no warnings 'redefine';
-   my $_pkg = caller;
 
    *{ $_pkg.'::mce_grep_f' } = \&run_file;
    *{ $_pkg.'::mce_grep_s' } = \&run_seq;
    *{ $_pkg.'::mce_grep'   } = \&run;
 
+   ## Process module arguments.
+   while ( my $_argument = shift ) {
+      my $_arg = lc $_argument;
+
+      $_p->{MAX_WORKERS} = shift, next if ( $_arg eq 'max_workers' );
+      $_p->{CHUNK_SIZE}  = shift, next if ( $_arg eq 'chunk_size' );
+      $_p->{TMP_DIR}     = shift, next if ( $_arg eq 'tmp_dir' );
+      $_p->{FREEZE}      = shift, next if ( $_arg eq 'freeze' );
+      $_p->{THAW}        = shift, next if ( $_arg eq 'thaw' );
+
+      ## Sereal 3.008+, if available, is used automatically by MCE 1.800.
+      if ( $_arg eq 'sereal' ) {
+         if ( shift eq '0' ) {
+            require Storable;
+            $_p->{FREEZE} = \&Storable::freeze;
+            $_p->{THAW}   = \&Storable::thaw;
+         }
+         next;
+      }
+
+      _croak("Error: ($_argument) invalid module option");
+   }
+
+   $_p->{MAX_WORKERS} = MCE::Util::_parse_max_workers($_p->{MAX_WORKERS});
+
+   _validate_number($_p->{MAX_WORKERS}, 'MAX_WORKERS');
+   _validate_number($_p->{CHUNK_SIZE}, 'CHUNK_SIZE')
+      unless ($_p->{CHUNK_SIZE} eq 'auto');
+
    return;
 }
 
-END {
-   return if (defined $_MCE && $_MCE->wid);
-
-   finish();
-}
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Gather callback for storing by chunk_id => chunk_ref into a hash.
@@ -121,27 +111,32 @@ sub _gather {
 sub init (@) {
 
    shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
+   my $_pkg = "$$.$_tid.".caller();
 
-   if (MCE->wid) {
-      @_ = (); _croak(
-         "$_tag: function cannot be called by the worker process"
-      );
-   }
-
-   finish(); $_params = (ref $_[0] eq 'HASH') ? shift : { @_ };
+   $_params->{$_pkg} = (ref $_[0] eq 'HASH') ? shift : { @_ };
 
    @_ = ();
 
    return;
 }
 
-sub finish () {
+sub finish (@) {
 
-   if (defined $_MCE && $_MCE->{_spawned}) {
-      MCE::_save_state; $_MCE->shutdown(); MCE::_restore_state;
+   shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
+   my $_pkg = (defined $_[0]) ? shift : "$$.$_tid.".caller();
+
+   if ( $_pkg eq 'MCE' ) {
+      for my $_k ( keys %{ $_MCE } ) { MCE::Grep->finish($_k, 1); }
+   }
+   elsif ( $_MCE->{$_pkg} && $_MCE->{$_pkg}{_init_pid} eq "$$.$_tid" ) {
+      $_MCE->{$_pkg}->shutdown(@_) if $_MCE->{$_pkg}{_spawned};
+      $_total_chunks = undef, undef %_tmp;
+
+      delete $_prev_c->{$_pkg};
+      delete $_MCE->{$_pkg};
    }
 
-   $_prev_c = $_total_chunks = undef; undef %_tmp;
+   @_ = ();
 
    return;
 }
@@ -157,23 +152,24 @@ sub run_file (&@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
 
    my $_code = shift; my $_file = shift;
+   my $_pid  = "$$.$_tid.".caller();
 
-   if (defined $_params) {
-      delete $_params->{input_data} if (exists $_params->{input_data});
-      delete $_params->{sequence}   if (exists $_params->{sequence});
+   if (defined (my $_p = $_params->{$_pid})) {
+      delete $_p->{input_data} if (exists $_p->{input_data});
+      delete $_p->{sequence}   if (exists $_p->{sequence});
    }
    else {
-      $_params = {};
+      $_params->{$_pid} = {};
    }
 
    if (defined $_file && ref $_file eq '' && $_file ne '') {
-      _croak("$_tag: ($_file) does not exist") unless (-e $_file);
-      _croak("$_tag: ($_file) is not readable") unless (-r $_file);
+      _croak("$_tag: ($_file) does not exist")      unless (-e $_file);
+      _croak("$_tag: ($_file) is not readable")     unless (-r $_file);
       _croak("$_tag: ($_file) is not a plain file") unless (-f $_file);
-      $_params->{_file} = $_file;
+      $_params->{$_pid}{_file} = $_file;
    }
-   elsif (ref $_file eq 'GLOB' || ref $_file eq 'SCALAR' || ref($_file) =~ /^IO::/) {
-      $_params->{_file} = $_file;
+   elsif (ref $_file eq 'SCALAR' || ref($_file) =~ /^(?:GLOB|FileHandle|IO::)/) {
+      $_params->{$_pid}{_file} = $_file;
    }
    else {
       _croak("$_tag: (file) is not specified or valid");
@@ -195,28 +191,29 @@ sub run_seq (&@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
 
    my $_code = shift;
+   my $_pid  = "$$.$_tid.".caller();
 
-   if (defined $_params) {
-      delete $_params->{input_data} if (exists $_params->{input_data});
-      delete $_params->{_file}      if (exists $_params->{_file});
+   if (defined (my $_p = $_params->{$_pid})) {
+      delete $_p->{input_data} if (exists $_p->{input_data});
+      delete $_p->{_file}      if (exists $_p->{_file});
    }
    else {
-      $_params = {};
+      $_params->{$_pid} = {};
    }
 
    my ($_begin, $_end);
 
    if (ref $_[0] eq 'HASH') {
       $_begin = $_[0]->{begin}; $_end = $_[0]->{end};
-      $_params->{sequence} = $_[0];
+      $_params->{$_pid}{sequence} = $_[0];
    }
    elsif (ref $_[0] eq 'ARRAY') {
       $_begin = $_[0]->[0]; $_end = $_[0]->[1];
-      $_params->{sequence} = $_[0];
+      $_params->{$_pid}{sequence} = $_[0];
    }
    elsif (ref $_[0] eq '') {
       $_begin = $_[0]; $_end = $_[1];
-      $_params->{sequence} = [ @_ ];
+      $_params->{$_pid}{sequence} = [ @_ ];
    }
    else {
       _croak("$_tag: (sequence) is not specified or valid");
@@ -224,11 +221,10 @@ sub run_seq (&@) {
 
    _croak("$_tag: (begin) is not specified for sequence")
       unless (defined $_begin);
-
    _croak("$_tag: (end) is not specified for sequence")
       unless (defined $_end);
 
-   $_params->{sequence_run} = 1;
+   $_params->{$_pid}{sequence_run} = 1;
 
    @_ = ();
 
@@ -245,21 +241,18 @@ sub run (&@) {
 
    shift if (defined $_[0] && $_[0] eq 'MCE::Grep');
 
-   my $_code = shift;   $_total_chunks = 0; undef %_tmp;
+   my $_code = shift;  $_total_chunks = 0; undef %_tmp;
+   my $_pkg  = caller() eq 'MCE::Grep' ? caller(1) : caller();
+   my $_pid  = "$$.$_tid.$_pkg";
 
-   if (MCE->wid) {
-      @_ = (); _croak(
-         "$_tag: function cannot be called by the worker process"
-      );
+   my $_input_data; my $_max_workers = $_def->{$_pkg}{MAX_WORKERS};
+   my $_r = ref $_[0];
+
+   if ($_r eq 'ARRAY' || $_r eq 'CODE' || $_r eq 'SCALAR' || $_r =~ /^(?:GLOB|FileHandle|IO::)/) {
+      $_input_data = shift if (@_ == 1);
    }
 
-   my $_input_data; my $_max_workers = $MAX_WORKERS; my $_r = ref $_[0];
-
-   if ($_r eq 'ARRAY' || $_r eq 'CODE' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
-      $_input_data = shift;
-   }
-
-   if (defined $_params) { my $_p = $_params;
+   if (defined (my $_p = $_params->{$_pid})) {
       $_max_workers = MCE::Util::_parse_max_workers($_p->{max_workers})
          if (exists $_p->{max_workers});
 
@@ -272,27 +265,27 @@ sub run (&@) {
    }
 
    my $_chunk_size = MCE::Util::_parse_chunk_size(
-      $CHUNK_SIZE, $_max_workers, $_params, $_input_data, scalar @_
+      $_def->{$_pkg}{CHUNK_SIZE}, $_max_workers, $_params->{$_pid},
+      $_input_data, scalar @_
    );
 
-   if (defined $_params) {
-      if (exists $_params->{_file}) {
-         $_input_data = delete $_params->{_file};
-      }
-      else {
-         $_input_data = $_params->{input_data} if exists $_params->{input_data};
+   if (defined (my $_p = $_params->{$_pid})) {
+      if (exists $_p->{_file}) {
+         $_input_data = delete $_p->{_file};
+      } else {
+         $_input_data = $_p->{input_data} if exists $_p->{input_data};
       }
    }
 
-   MCE::_save_state;
-
    ## -------------------------------------------------------------------------
 
-   if (!defined $_prev_c || $_prev_c != $_code) {
-      $_MCE->shutdown() if (defined $_MCE);
-      $_prev_c = $_code;
+   MCE::_save_state();
 
-      my %_options = (
+   if (!defined $_prev_c->{$_pid} || $_prev_c->{$_pid} != $_code) {
+      $_MCE->{$_pid}->shutdown() if (defined $_MCE->{$_pid});
+      $_prev_c->{$_pid} = $_code;
+
+      my %_opts = (
          max_workers => $_max_workers, task_name => $_tag,
          user_func => sub {
 
@@ -304,9 +297,11 @@ sub run (&@) {
 
                if (ref $_chunk_ref eq 'SCALAR') {
                   local $/ = $_mce->{RS} if defined $_mce->{RS};
-                  open my  $_MEM_FH, '<', $_chunk_ref;
-                  while ( <$_MEM_FH> ) { push (@_a, $_) if &{ $_code }; }
-                  close    $_MEM_FH;
+                  open my $_MEM_FH, '<', $_chunk_ref;
+                  binmode $_MEM_FH, ':raw';
+                  while (<$_MEM_FH>) { push (@_a, $_) if &{ $_code }; }
+                  close   $_MEM_FH;
+                  weaken  $_MEM_FH;
                }
                else {
                   if (ref $_chunk_ref) {
@@ -323,9 +318,11 @@ sub run (&@) {
 
                if (ref $_chunk_ref eq 'SCALAR') {
                   local $/ = $_mce->{RS} if defined $_mce->{RS};
-                  open my  $_MEM_FH, '<', $_chunk_ref;
-                  while ( <$_MEM_FH> ) { $_cnt++ if &{ $_code }; }
-                  close    $_MEM_FH;
+                  open my $_MEM_FH, '<', $_chunk_ref;
+                  binmode $_MEM_FH, ':raw';
+                  while (<$_MEM_FH>) { $_cnt++ if &{ $_code }; }
+                  close   $_MEM_FH;
+                  weaken  $_MEM_FH;
                }
                else {
                   if (ref $_chunk_ref) {
@@ -340,61 +337,61 @@ sub run (&@) {
          },
       );
 
-      if (defined $_params) {
-         for my $_p (keys %{ $_params }) {
-            next if ($_p eq 'sequence_run');
-            next if ($_p eq 'input_data');
-            next if ($_p eq 'chunk_size');
+      if (defined (my $_p = $_params->{$_pid})) {
+         for my $_k (keys %{ $_p }) {
+            next if ($_k eq 'sequence_run');
+            next if ($_k eq 'input_data');
+            next if ($_k eq 'chunk_size');
 
-            _croak("MCE::Grep: ($_p) is not a valid constructor argument")
-               unless (exists $MCE::_valid_fields_new{$_p});
+            _croak("MCE::Grep: ($_k) is not a valid constructor argument")
+               unless (exists $MCE::_valid_fields_new{$_k});
 
-            $_options{$_p} = $_params->{$_p};
+            $_opts{$_k} = $_p->{$_k};
          }
       }
 
-      $_MCE = MCE->new(%_options);
+      for my $_k (qw/ tmp_dir freeze thaw /) {
+         $_opts{$_k} = $_def->{$_pkg}{uc($_k)}
+            if (exists $_def->{$_pkg}{uc($_k)} && !exists $_opts{$_k});
+      }
+
+      $_MCE->{$_pid} = MCE->new(pkg => $_pkg, %_opts);
    }
 
    ## -------------------------------------------------------------------------
 
    my $_cnt = 0; my $_wantarray = wantarray;
 
-   $_MCE->{use_slurpio} = ($_chunk_size > MCE::MAX_RECS_SIZE) ? 1 : 0;
-   $_MCE->{user_args} = [ $_wantarray ];
+   $_MCE->{$_pid}{use_slurpio} = ($_chunk_size > &MCE::MAX_RECS_SIZE) ? 1 : 0;
+   $_MCE->{$_pid}{user_args}   = [ $_wantarray ];
 
-   $_MCE->{gather} = $_wantarray
+   $_MCE->{$_pid}{gather} = $_wantarray
       ? \&_gather : sub { $_cnt += $_[0]; return; };
 
    if (defined $_input_data) {
       @_ = ();
-      $_MCE->process({ chunk_size => $_chunk_size }, $_input_data);
-      delete $_MCE->{input_data};
+      $_MCE->{$_pid}->process({ chunk_size => $_chunk_size }, $_input_data);
+      delete $_MCE->{$_pid}{input_data};
    }
    elsif (scalar @_) {
-      $_MCE->process({ chunk_size => $_chunk_size }, \@_);
-      delete $_MCE->{input_data};
+      $_MCE->{$_pid}->process({ chunk_size => $_chunk_size }, \@_);
+      delete $_MCE->{$_pid}{input_data};
    }
    else {
-      if (defined $_params && exists $_params->{sequence}) {
-         $_MCE->run({
-            chunk_size => $_chunk_size, sequence => $_params->{sequence}
+      if (defined $_params->{$_pid} && exists $_params->{$_pid}{sequence}) {
+         $_MCE->{$_pid}->run({
+             chunk_size => $_chunk_size,
+             sequence   => $_params->{$_pid}{sequence}
          }, 0);
-         if (exists $_params->{sequence_run}) {
-            delete $_params->{sequence_run};
-            delete $_params->{sequence};
+         if (exists $_params->{$_pid}{sequence_run}) {
+             delete $_params->{$_pid}{sequence_run};
+             delete $_params->{$_pid}{sequence};
          }
-         delete $_MCE->{sequence};
+         delete $_MCE->{$_pid}{sequence};
       }
    }
 
-   MCE::_restore_state;
-
-   if (exists $_MCE->{_rla_return}) {
-      $MCE::MCE->{_rla_return} = delete $_MCE->{_rla_return};
-   }
-
-   finish() if ($^S);   ## shutdown if in eval state
+   MCE::_restore_state();
 
    if ($_wantarray) {
       return map { @{ $_ } } delete @_tmp{ 1 .. $_total_chunks };
@@ -448,7 +445,7 @@ MCE::Grep - Parallel grep model similar to the native grep function
 
 =head1 VERSION
 
-This document describes MCE::Grep version 1.608
+This document describes MCE::Grep version 1.810
 
 =head1 SYNOPSIS
 
@@ -498,10 +495,9 @@ otherwise -1.
 
    my @m3 = mce_grep_s { /[2357][1468][9]/ } 1, 1000000;  ## 0.165 secs
 
-Although this document is about MCE::Grep, the L<MCE::Stream|MCE::Stream>
-module can write results immediately without waiting for all chunks to
-complete. This is made possible by passing the reference to an array
-(in this case @m4 and @m5).
+Although this document is about MCE::Grep, the L<MCE::Stream> module can write
+results immediately without waiting for all chunks to complete. This is made
+possible by passing the reference to an array (in this case @m4 and @m5).
 
    use MCE::Stream default_mode => 'grep';
 
@@ -540,30 +536,75 @@ Testing was done against a 300 MB file containing 250k lines.
    @m = mce_grep_f { /pattern/ } $LOG;                  ##  0.112 secs
    @m = mce_grep_f { /foobar|[2357][1468][9]/ } $LOG;   ##  6.840 secs
 
+=head1 PARSING HUGE FILES
+
+The MCE::Grep module lacks an optimization for quickly determining if a match
+is found from not knowing the pattern inside the code block. Use the following
+snippet as a template to achieve better performance. Also, take a look at
+examples/egrep.pl, included with the distribution.
+
+   use MCE::Loop;
+
+   MCE::Loop::init {
+      max_workers => 8, use_slurpio => 1
+   };
+
+   my $pattern  = 'karl';
+   my $hugefile = 'very_huge.file';
+
+   my @result = mce_loop_f {
+      my ($mce, $slurp_ref, $chunk_id) = @_;
+
+      ## Quickly determine if a match is found.
+      ## Process slurped chunk only if true.
+
+      if ($$slurp_ref =~ /$pattern/m) {
+         my @matches;
+
+         ## The following is fast on Unix. Performance degrades
+         ## drastically on Windows beyond 4 workers.
+
+         open my $MEM_FH, '<', $slurp_ref;
+         binmode $MEM_FH, ':raw';
+         while (<$MEM_FH>) { push @matches, $_ if (/$pattern/); }
+         close   $MEM_FH;
+
+         ## Therefore, use the following construct on Windows.
+
+         while ( $$slurp_ref =~ /([^\n]+\n)/mg ) {
+            my $line = $1; # save $1 to not lose the value
+            push @matches, $line if ($line =~ /$pattern/);
+         }
+
+         ## Gather matched lines.
+
+         MCE->gather(@matches);
+      }
+
+   } $hugefile;
+
+   print join('', @result);
+
 =head1 OVERRIDING DEFAULTS
 
-The following list 5 options which may be overridden when loading the module.
+The following list options which may be overridden when loading the module.
 
    use Sereal qw( encode_sereal decode_sereal );
    use CBOR::XS qw( encode_cbor decode_cbor );
    use JSON::XS qw( encode_json decode_json );
 
    use MCE::Grep
-         max_workers => 4,               ## Default 'auto'
-         chunk_size => 100,              ## Default 'auto'
-         tmp_dir => "/path/to/app/tmp",  ## $MCE::Signal::tmp_dir
-         freeze => \&encode_sereal,      ## \&Storable::freeze
-         thaw => \&decode_sereal         ## \&Storable::thaw
+       max_workers => 4,                # Default 'auto'
+       chunk_size => 100,               # Default 'auto'
+       tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
+       freeze => \&encode_sereal,       # \&Storable::freeze
+       thaw => \&decode_sereal          # \&Storable::thaw
    ;
 
-There is a simpler way to enable Sereal with MCE 1.5. The following will
-attempt to use Sereal if available, otherwise defaults to Storable for
-serialization.
+From MCE 1.8 onwards, Sereal 3.008+ is loaded automatically if available.
+Specify C<Sereal => 0> to use Storable instead.
 
-   use MCE::Grep Sereal => 1;
-
-   ## Serialization is by the Sereal module if available.
-   my @m2 = mce_grep { $_ % 5 == 0 } 1..10000;
+   use MCE::Grep Sereal => 0;
 
 =head1 CUSTOMIZING MCE
 
@@ -618,7 +659,7 @@ specified, is ignored due to being used internally by the module.
 =item mce_grep { code } iterator
 
 An iterator reference can by specified for input_data. Iterators are described
-under "SYNTAX for INPUT_DATA" at L<MCE::Core|MCE::Core>.
+under "SYNTAX for INPUT_DATA" at L<MCE::Core>.
 
    my @a = mce_grep { $_ % 3 == 0 } make_iterator(10, 30, 2);
 
@@ -687,7 +728,7 @@ longer needed.
 
 =head1 INDEX
 
-L<MCE|MCE>
+L<MCE|MCE>, L<MCE::Core>
 
 =head1 AUTHOR
 
